'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (cwd) {
  var pkgPath = (0, _path.join)(cwd, 'package.json');
  var pkg = {};
  if ((0, _fs.existsSync)(pkgPath)) {
    delete require.cache[require.resolve(pkgPath)];
    pkg = require(pkgPath); // eslint-disable-line
  }

  return {
    mode: 'development',
    entry: pkg.entry,
    devtool: '#cheap-module-source-map',
    output: {
      path: (0, _path.join)(cwd, 'dist'),
      filename: '[name].js',
      publicPath: '/'
    },
    resolve: {
      // 优先从自身去 resolve，自身 resolve 不到时，则需要到父目录 resolve
      modules: ['node_modules', (0, _path.resolve)(__dirname, '../../node_modules'), (0, _path.resolve)(__dirname, '../../../../')],
      extensions: ['.js', '.jsx', '.json', '.less', '.css'],
      mainFields: ['main']
    },
    resolveLoader: {
      modules: ['node_modules', (0, _path.resolve)(__dirname, '../../node_modules'), (0, _path.resolve)(__dirname, '../../../../')],
      moduleExtensions: ['-loader']
    },
    module: {
      rules: [{
        oneOf: [{
          test: /\.(js|jsx)$/,
          exclude: /node_modules/,
          use: [{
            loader: require.resolve('babel-loader'),
            options: babelOptions
          },
          // 注入组件名字
          {
            loader: require.resolve('fengdie-component-loader/lib/nameInjectLoader'),
            options: {
              cwd: cwd
            }
          },
          // 自动替换 import 可复用组件为 window.fn
          {
            loader: require.resolve('fengdie-component-loader/lib/importReplaceLoader'),
            options: {
              components: (0, _fengdieComponentLoader.components)(cwd).join(';')
            }
          }]
        }, {
          test: /\.css$/,
          use: [_miniCssExtractPlugin2.default.loader, {
            loader: require.resolve('css-loader'),
            options: {
              importLoaders: 1
            }
          }, {
            loader: require.resolve('postcss-loader'),
            options: postcssOptions
          }]
        }, {
          test: /\.less/,
          use: [_miniCssExtractPlugin2.default.loader, {
            loader: require.resolve('css-loader'),
            options: {
              importLoaders: 2
            }
          }, {
            loader: require.resolve('postcss-loader'),
            options: postcssOptions
          }, require.resolve('less-loader')]
        }, {
          test: /\.html$/,
          use: [{
            loader: require.resolve('file-loader'),
            options: {
              name: '[name].[ext]'
            }
          }]
        }, {
          test: /\.(svg)$/i,
          use: [require.resolve('svg-sprite-loader')]
        }, {
          exclude: [/\.html$/, /\.(js|jsx)$/, /\.(css|less)$/, /\.json$/, /\.svg$/],
          use: [{
            loader: require.resolve('url-loader'),
            options: {
              limit: 10000,
              name: 'static/[name].[hash:8].[ext]'
            }
          }]
        }]
      }]
    },
    plugins: [new _caseSensitivePathsWebpackPlugin2.default(), new _webpack2.default.DefinePlugin({
      'process.env': {
        NODE_ENV: JSON.stringify(process.env.NODE_ENV)
      }
    }), new _miniCssExtractPlugin2.default({
      filename: '[name].css'
    })],
    externals: {
      react: 'window.React',
      'react-dom': 'window.ReactDOM'
    },
    node: {
      fs: 'empty',
      net: 'empty',
      tls: 'empty'
    }
  };
};

var _path = require('path');

var _fs = require('fs');

var _miniCssExtractPlugin = require('mini-css-extract-plugin');

var _miniCssExtractPlugin2 = _interopRequireDefault(_miniCssExtractPlugin);

var _caseSensitivePathsWebpackPlugin = require('case-sensitive-paths-webpack-plugin');

var _caseSensitivePathsWebpackPlugin2 = _interopRequireDefault(_caseSensitivePathsWebpackPlugin);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _fengdieComponentLoader = require('fengdie-component-loader');

var _webpack = require('ide-webpack-common/lib/webpack');

var _webpack2 = _interopRequireDefault(_webpack);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-line

var postcssOptions = {
  ident: 'postcss',
  plugins: function plugins() {
    return [(0, _autoprefixer2.default)({
      browsers: ['Android >= 4.0', 'iOS >= 8']
    })];
  }
};

var babelOptions = {
  babelrc: false,
  presets: [require.resolve('babel-preset-es2015'), require.resolve('babel-preset-react'), require.resolve('babel-preset-stage-0')],
  plugins: [[require.resolve('babel-plugin-transform-runtime'), {
    polyfill: false
  }], require.resolve('babel-plugin-add-module-exports')],
  cacheDirectory: true
};

module.exports = exports['default'];