'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = start;

var _fs = require('fs');

var _path = require('path');

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _ideUtil = require('ide-util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//eslint-disable-line

var originalPkgJSON = null;

function readPackage(pkgPath) {
  var pkgJSON = null;
  try {
    pkgJSON = JSON.parse((0, _fs.readFileSync)(pkgPath));
  } catch (e) {
    console.log('read ' + pkgPath + ' failed', e);
  }
  return pkgJSON;
}

function isChange(updatedPkgJson, item) {
  var newValue = updatedPkgJson[item] || {};
  var oldValue = originalPkgJSON[item] || {};

  // 如果两者数量不一致，则更新
  if (Object.keys(newValue).length !== Object.keys(oldValue).length) return true;

  // 如果两者 key,val 不一致，则更新
  // eslint-disable-next-line
  for (var variable in newValue) {
    if (newValue[variable] !== oldValue[variable]) {
      return true;
    }
  }
  return false;
}

function isEntryOrDepsUpdated(updatedPkgJson) {
  if (!updatedPkgJson) return false; // 如果最新的 pkgJson 读取不到或者非法，什么都不更新
  if (!originalPkgJSON) return true; // 如果旧版的 pkgJson 不存在，而新版的存在，则更新

  return isChange(updatedPkgJson, 'entry') || isChange(updatedPkgJson, 'dependencies') || isChange(updatedPkgJson, 'devDependencies');
}

function start() {
  var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var cwd = args.cwd || process.cwd();
  var pkg = (0, _path.join)(cwd, 'package.json');
  originalPkgJSON = readPackage(pkg);

  var watcher = _chokidar2.default.watch([pkg, (0, _path.join)(cwd, 'webpack.config.js')], {
    ignored: /\.idea|\.tea/,
    persistent: true
  });
  watcher.on('all', function (event, path) {
    if (['change', 'unlink'].indexOf(event) !== -1) {
      console.log(path + ' changed');
      if (path.indexOf('package.json') === -1) {
        // webpack.config.js 的变化
        (0, _ideUtil.send)({ type: _ideUtil.ON_ENTRY_UPDATED, isUpdate: true });
        return;
      }
      var updatedPkgJson = readPackage(path);
      var isUpdate = isEntryOrDepsUpdated(updatedPkgJson);
      if (updatedPkgJson) originalPkgJSON = updatedPkgJson;

      console.log(path + ' entry isUpdate: ' + isUpdate);
      (0, _ideUtil.send)({ type: _ideUtil.ON_ENTRY_UPDATED, isUpdate: isUpdate });
    }
  });

  process.on('SIGINT', function () {
    watcher.close();
  });
}
module.exports = exports['default'];