'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = pack;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _ideUtil = require('ide-util');

var _schemaUtil = require('schema-util');

var _schemaUtil2 = _interopRequireDefault(_schemaUtil);

var _vinylFs = require('vinyl-fs');

var _vinylFs2 = _interopRequireDefault(_vinylFs);

var _mapStream = require('map-stream');

var _mapStream2 = _interopRequireDefault(_mapStream);

var _gulpTar = require('gulp-tar');

var _gulpTar2 = _interopRequireDefault(_gulpTar);

var _gulpGzip = require('gulp-gzip');

var _gulpGzip2 = _interopRequireDefault(_gulpGzip);

var _gulpFilter = require('gulp-filter');

var _gulpFilter2 = _interopRequireDefault(_gulpFilter);

var _gulpFile = require('gulp-file');

var _gulpFile2 = _interopRequireDefault(_gulpFile);

var _gulpUtil = require('gulp-util');

var _gulpUtil2 = _interopRequireDefault(_gulpUtil);

var _gulpRename = require('gulp-rename');

var _gulpRename2 = _interopRequireDefault(_gulpRename);

var _gulpChmod = require('gulp-chmod');

var _gulpChmod2 = _interopRequireDefault(_gulpChmod);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 当前 cwd
var cwd = void 0;
// fengdie.config.js 和 package.json mix 后的 package.json
// eslint-disable-line
var newPkg = void 0;
// 记录是否是首次开始校验
var initialChecking = true;
// 记录是否首次开始打包
var initialPacking = false;
// ora
var spinner = void 0;

function prepare() {
  var pkg = (0, _utils.getPackageJson)(cwd);
  var config = (0, _utils.getFengdieConfig)(cwd);
  spinner.text = 'Preparing...';
  (0, _ideUtil.send)({ type: _ideUtil.PREPARING });

  if (config && config.title) {
    pkg.title = config.title;
  }
  config.title = pkg.title;
  // fengdie.config.js 到一份 package.json 文件
  pkg.fengdie = config;

  return pkg;
}

function isEmpty(config, key) {
  return !(config[key] && config[key].trim());
}

function validatePackageJson() {
  var error = [];
  if (isEmpty(newPkg, 'name')) {
    error.push('包名 name 为必填项');
  }
  if (!/^[a-zA-Z][\w/-]*$/i.test(newPkg.name)) {
    error.push('包名 name 必须是由英文字符、数字、/、-这几个字符组成，并以英文字符开头');
  }
  if (isEmpty(newPkg, 'version')) {
    error.push('包版本号 version 为必填项');
  }
  if (!_fs2.default.existsSync((0, _path.join)(cwd, 'snapshot.png'))) {
    error.push('缺少包缩略图 snapshot.png');
  }

  return error;
}

function validateSchema(f) {
  var text = f.contents.toString();
  var schemaFilePath = f.path;
  var error = [];
  try {
    var schema = void 0;
    try {
      // 支持最多 3 级嵌套
      schema = _schemaUtil2.default.schema(text, 3);
    } catch (e) {
      throw new Error('schema \u5B9A\u4E49\u6709\u8BEF\uFF1A' + schemaFilePath + '\uFF0C' + e.message);
    }
    // 1.2 mock 数据校验，同 schema 名的 .json 文件认为是 mock 数据
    var jsonPath = schemaFilePath.replace(/\.schema$/, '.json');
    // 如果存在 schema 对应的数据文件，检查数据是否正确
    if (_fs2.default.existsSync(jsonPath)) {
      var mock = _schemaUtil2.default.mock(schema);
      var json = void 0;
      try {
        json = JSON.parse(_fs2.default.readFileSync(jsonPath).toString());
      } catch (e) {
        throw new Error('json \u6570\u636E\u6709\u8BEF\uFF1A' + jsonPath);
      }
      var isCompatible = _schemaUtil2.default.merge.isCompatible(mock, json);
      if (!isCompatible) {
        throw new Error('mock \u6570\u636E\u548C schema \u5B9A\u4E49\u4E0D\u4E00\u81F4\uFF1A' + jsonPath);
      }
    }
  } catch (e) {
    error.push(e.message);
  }

  return error;
}

function validate(f, callback) {
  if (initialChecking) {
    spinner.text = '开始校验 package.json 和 相关 schema';
    (0, _ideUtil.send)({ type: _ideUtil.CHECKING });
  }
  if (!f.isDirectory()) {
    if (f.path === 'package.json') {
      initialPacking = true;
      var pkgErr = validatePackageJson();
      if (pkgErr.length) {
        callback(new _gulpUtil2.default.PluginError('validator:checking:package.json', new Error(pkgErr.join('\n'))));
      }
    } else if (f.extname === '.schema') {
      initialPacking = true;
      var schemaErr = validateSchema(f);
      if (schemaErr.length) {
        callback(new _gulpUtil2.default.PluginError('validator:checking:*.schema', new Error(schemaErr.join('\n'))));
      }
    }
  }
  // 保证只输出一次
  if (initialChecking && initialPacking) {
    spinner.text = '开始构建相关压缩包';
    (0, _ideUtil.send)({ type: _ideUtil.PACKING });
  }
  initialChecking = false;
  callback(null, f);
}

function pack() {
  var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  spinner = (0, _ideUtil.ora)('进入打包流程').start();
  cwd = args.cwd || process.cwd();
  newPkg = prepare();

  var packFileName = (0, _utils.getTarFileName)(newPkg);
  var fengdieDir = (0, _utils.getFengdieDir)();
  var tarFile = (0, _path.join)(fengdieDir, packFileName);
  // 删除上次打包的文件，避免发布失败后文件被重复打包的问题
  _ideUtil.efs.removeSync(tarFile);
  // 过滤获取需要校验的文件
  var validatorFilter = (0, _gulpFilter2.default)(['**/*.schema', 'package.json', '!dist/**/*'], { restore: true });

  return new Promise(function (resolve, reject) {
    function packErrorCallback(err) {
      if (err) {
        var plugin = err.plugin || 'ERROR:';
        var message = err.message || '未知出错信息';
        _gulpUtil2.default.log(_gulpUtil2.default.colors.red(plugin + '\n' + message));
        (0, _ideUtil.send)({ type: _ideUtil.PACK_DONE, success: false });
        reject({ err: err, spinner: spinner });
      }
    }
    _vinylFs2.default.src(['**/*', '!./package.json'].concat(_utils.GLOB_IGNORES), { cwd: cwd, dot: true }).pipe((0, _gulpFile2.default)('package.json', JSON.stringify(newPkg, null, 2))).pipe(validatorFilter).pipe((0, _mapStream2.default)(validate)).on('error', packErrorCallback).pipe(validatorFilter.restore).pipe((0, _gulpRename2.default)(function (path) {
      path.dirname = 'package/' + path.dirname; //eslint-disable-line
    })).pipe((0, _gulpChmod2.default)(511, true)).pipe((0, _gulpTar2.default)(packFileName)).on('error', packErrorCallback).pipe((0, _gulpGzip2.default)({ append: false })).on('error', packErrorCallback).pipe(_vinylFs2.default.dest(fengdieDir)).on('error', packErrorCallback).on('end', function () {
      (0, _ideUtil.send)({ type: _ideUtil.PACK_DONE, success: true, tarFile: tarFile });

      resolve({ tarFile: tarFile, spinner: spinner });
    });
  });
}
module.exports = exports['default'];