'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = publish;

var _path = require('path');

var _fs = require('fs');

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _ideUtil = require('ide-util');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var akskPath = (0, _path.join)((0, _utils.getFengdieDir)(), '.fengdie_ak_sk'); // eslint-disable-line

var spinner = void 0;

function getAKSK(force) {
  spinner.text = '获取 AK SK';
  var questions = [{
    type: 'input',
    name: 'ak',
    message: "What's your access key, you can get it from https://fengdie.alipay-eco.com/package/settings"
  }, {
    type: 'input',
    name: 'sk',
    message: "What's your secret key, you can get it from https://fengdie.alipay-eco.com/package/settings"
  }];
  var aksk = void 0;
  // 如果 aksk 已被缓存且并不需要强制获取则从本地缓存文件中读取
  if ((0, _fs.existsSync)(akskPath) && !force) {
    try {
      var akskCache = JSON.parse((0, _fs.readFileSync)(akskPath, 'utf8'));
      if (akskCache.ak && akskCache.sk) {
        aksk = Promise.resolve(akskCache);
      } else {
        throw new Error('ERROR: Failed to load ' + akskPath);
      }
    } catch (e) {
      aksk = _inquirer2.default.prompt(questions);
    }
  }
  // 如果 aksk 不存在或者需要强制获取则无论如何需要用户再次输入 ak sk
  if (!(0, _fs.existsSync)(akskPath) || force) {
    aksk = _inquirer2.default.prompt(questions);
  }

  return aksk;
}

function sign(params, secretKey) {
  return _crypto2.default.createHmac('sha1', secretKey).update(params).digest().toString('base64');
}

function publishToServer(tarFile, aksk) {
  spinner.info('Access Key: ' + aksk.ak + ' Secret Key: ' + aksk.sk);
  var fileStr = (0, _fs.readFileSync)(tarFile);
  var params = JSON.stringify({
    md5: (0, _md2.default)(fileStr),
    accessKey: aksk.ak
  });
  var url = (0, _utils.getRemote)(null, '/api/v1/package') + '&ak=' + aksk.ak;

  spinner.text = '进入 publish 流程';

  return new Promise(function (resolve, reject) {
    _ideUtil.superagent.post(url).type('form').set('x-fengdie-params', params).set('x-fengdie-sign', sign(params, aksk.sk)).field('file', (0, _fs.createReadStream)(tarFile)).end(function (err, res) {
      if (err) {
        reject(err);

        return;
      }
      var ret = res.body || '';
      if (ret.success) {
        resolve(ret);
        (0, _fs.writeFileSync)(akskPath, JSON.stringify(aksk, null, 2));
      } else {
        var errorMessage = ret.errorMsg || '推送失败::未知出错::';
        reject(errorMessage);
      }
    });
  });
}

function publish() {
  var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  spinner = (0, _ideUtil.ora)('进入 publish 流程').start();

  return new Promise(function (resolve, reject) {
    getAKSK(!!args.force).then(function (aksk) {
      publishToServer(args.tarFile, aksk).then(function (ret) {
        resolve({ ret: ret, spinner: spinner });
      }).catch(function (err) {
        reject({ err: err, spinner: spinner });
      });
    }).catch(function (err) {
      reject({ err: err, spinner: spinner });
    });
  });
}
module.exports = exports['default'];