'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GLOB_IGNORES = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // eslint-disable-line


exports.getFengdieConfig = getFengdieConfig;
exports.getPackageJson = getPackageJson;
exports.getTarFileName = getTarFileName;
exports.getWebpackConfig = getWebpackConfig;
exports.getFengdieDir = getFengdieDir;
exports.installDeps = installDeps;
exports.getRemote = getRemote;

var _path = require('path');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _child_process = require('child_process');

var _ideUtil = require('ide-util');

var _unzip = require('unzip');

var _unzip2 = _interopRequireDefault(_unzip);

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _webpackConfig = require('./config/webpack.config.dev');

var _webpackConfig2 = _interopRequireDefault(_webpackConfig);

var _webpackConfig3 = require('./config/webpack.config.prod');

var _webpackConfig4 = _interopRequireDefault(_webpackConfig3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FD_ENV_MAP = {
  alipay: 'https://fengdie.alipay.com',
  antcloud: 'https://api.yunfengdie.com'
};

var DEFAULT_FENGDIE_CONFIG = {
  type: 'template',
  renderType: 'RENDER'
};

function getFengdieConfig(cwd) {
  var config = {};
  try {
    var fengdieConfigPath = (0, _path.join)(cwd || process.cwd(), 'fengdie.config.js');
    delete require.cache[require.resolve(fengdieConfigPath)];
    config = require(fengdieConfigPath); // eslint-disable-line
  } catch (e) {
    // 如果不存在 fengdie.config.js 默认提供凤蝶模板包配置
  }
  return _extends({}, DEFAULT_FENGDIE_CONFIG, config);
}

function getPackageJson(cwd) {
  try {
    var pkgPath = (0, _path.join)(cwd || process.cwd(), 'package.json');
    delete require.cache[require.resolve(pkgPath)];
    return require(pkgPath); // eslint-disable-line
  } catch (e) {
    throw e;
  }
}

// 获取压缩包的文件名
function getTarFileName(pkg) {
  var name = pkg.name.replace(/^@/, '').replace('/', '-');
  return name + '-' + pkg.version + '.tgz';
}

function getWebpackConfig(type, cwd) {
  var defaultConfig = void 0;
  if (type === 'dev') {
    defaultConfig = (0, _webpackConfig2.default)(cwd);
  } else {
    defaultConfig = (0, _webpackConfig4.default)(cwd);
  }

  // 如果当前目录下有 webpack.config.js 优先使用
  var customWebpackPath = (0, _path.join)(cwd, 'webpack.config.js');
  if (_fs2.default.existsSync(customWebpackPath)) {
    console.warn(_ideUtil.chalk.yellow('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'));
    console.warn(_ideUtil.chalk.yellow('使用 webpack.config.js 扩展构建的配置的方式存在高危性，后续将提供更好的扩展方式！'));
    console.warn(_ideUtil.chalk.yellow('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'));
    delete require.cache[require.resolve(customWebpackPath)];
    return require(customWebpackPath)(defaultConfig, cwd); // eslint-disable-line
  }
  return defaultConfig;
}

// /home/user/.fengdie
function getFengdieDir() {
  var base = (0, _path.join)(_os2.default.homedir() /* homeOrTmp */, '.fengdie');

  if (!_fs2.default.existsSync(base)) {
    (0, _ideUtil.mkdirp)(base);
  }
  return base;
}

function findCnpm() {
  return new Promise(function (resolve, reject) {
    var isWin32 = _os2.default.platform() === 'win32';
    var fengdieDir = getFengdieDir();
    var cnpmPath = (0, _path.join)(fengdieDir, isWin32 ? 'cnpm.exe' : 'cnpm');
    if (_fs2.default.existsSync(cnpmPath)) {
      resolve(cnpmPath);
      return;
    }
    var cnpmUrl = isWin32 ? 'https://gw.alipayobjects.com/os/enclose-prod/8f3ddbce-283e-4955-a4ca-b7d99f5a910c/cnpm-5.0.0-x64.zip' : 'https://gw.alipayobjects.com/os/enclose-prod/7c3b80ff-0fa4-428d-a0da-29f81d53c3ed/cnpm-5.0.0-darwin-x64.gz';
    console.log('Downloading cnpm from ' + cnpmUrl);
    console.log();

    var cnpmZipPath = isWin32 ? (0, _path.join)(fengdieDir, 'cnpm.zip') : (0, _path.join)(fengdieDir, 'cnpm.gz');
    var stream = _fs2.default.createWriteStream(cnpmZipPath);
    var resStatus = void 0;
    _ideUtil.superagent.get(cnpmUrl).timeout({
      deadline: 300000 // 五分钟
    }).on('error', function (err) {
      console.log('Download cnpm with error: ' + err);
      reject();
    }).on('response', function (res) {
      resStatus = res.status;
    }).pipe(stream).on('error', function (err) {
      console.log('Download cnpm with error: ' + err);
      reject();
    }).on('finish', function () {
      if (resStatus !== 200) {
        console.log('Download cnpm with error: ' + resStatus);
        reject();
        return;
      }

      if (isWin32) {
        _fs2.default.createReadStream(cnpmZipPath).pipe(_unzip2.default.Parse()).on('error', function (err) {
          console.log('Unzip cnpm.zip with error: ' + err);
          reject();
        }).on('entry', function (entry) {
          entry.pipe(_fs2.default.createWriteStream(cnpmPath)).on('finish', function () {
            console.log(_ideUtil.chalk.green('Cnpm.exe is Ready!'));
            console.log();
            resolve(cnpmPath);
          });
        });
      } else {
        _fs2.default.createReadStream(cnpmZipPath).pipe(_zlib2.default.createGunzip()).on('error', function (err) {
          console.log('Unzip cnpm.gz with error: ' + err);
          reject();
        }).pipe(_fs2.default.createWriteStream(cnpmPath)).on('finish', function () {
          // 给可执行包加上可执行权限
          _fs2.default.chmodSync(cnpmPath, '755');
          console.log(_ideUtil.chalk.green('Cnpm is Ready!'));
          console.log();
          resolve(cnpmPath);
        });
      }
    });
  });
}

function runCmd(cmd) {
  var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var cwd = arguments[2];
  var fn = arguments[3];

  var runner = (0, _child_process.spawn)(cmd, args, {
    stdio: 'inherit',
    cwd: cwd,
    env: _extends({}, process.env, {
      CI: true
    })
  });
  runner.on('close', function (code) {
    if (fn) {
      fn(code);
    }
  });
}

// 在 cwd 执行 cnpm i
function installDeps(cwd) {
  return new Promise(function (resolve, reject) {
    findCnpm().then(function (cnpmPath) {
      console.log('Installing project dependencies...');
      console.log();
      runCmd(cnpmPath, ['install'], cwd, function (code) {
        console.log('Install end with ' + code);
        console.log();
        resolve();
      });
    }, reject).catch(function (err) {
      console.log(err);
      reject();
    });
  });
}

// 获取凤蝶服务器地址
function getRemote(cwd, api, env) {
  var fengdiePackage = require('../package.json'); // eslint-disable-line
  // 目前只支持蚂蚁云上凤蝶，主站凤蝶支持需要打通登录体系
  var remote = FD_ENV_MAP.antcloud;
  if (cwd) {
    var prop = getFengdieConfig(cwd);
    remote = FD_ENV_MAP[env || prop.env || 'antcloud'];
    // registry 的优先级最高
    if (prop.registry) {
      remote = prop.registry;
    }
  }
  var v = fengdiePackage.version;

  return '' + remote.replace(/\/$/gi, '') + (api || '') + '?tinyCliVersion=' + v + '&hostname=' + _os2.default.hostname() + '&fdAK=' + (process.env.FDAK || '');
}

var GLOB_IGNORES = exports.GLOB_IGNORES = ['!.git', '!.git/**', '!.svn', '!.svn/**', '!node_modules', '!node_modules/**', '!spm_modules', '!spm_modules/**', '!.tea/**', '!.idea/**', '!.cache/**', '!.DS_Store', '!**/.DS_Store', '!**/.gitignore', '!**/.npmignore'];